@@ -386,7 +386,7 @@ func SaveConfig() {
 	ctx := context.Background()
 	ci := fs.GetConfig(ctx)
 	var err error
-	for i := 0; i < ci.LowLevelRetries+1; i++ {
+	for range ci.LowLevelRetries + 1 {
 		if err = LoadedData().Save(); err == nil {
 			return
 		}
@@ -514,6 +514,8 @@ type UpdateRemoteOpt struct {
 	Obscure bool `json:"obscure"`
 	// Treat all passwords as obscured
 	NoObscure bool `json:"noObscure"`
+	// Don't provide any output
+	NoOutput bool `json:"noOutput"`
 	// Don't interact with the user - return questions
 	NonInteractive bool `json:"nonInteractive"`
 	// If set then supply state and result parameters to continue the process
@@ -762,7 +764,7 @@ func SetCacheDir(path string) (err error) {
 //
 // To override the default we therefore set environment variable TMPDIR
 // on Unix systems, and both TMP and TEMP on Windows (they are almost exclusively
-// aliases for the same path, and programs may refer to to either of them).
+// aliases for the same path, and programs may refer to either of them).
 // This should make all libraries and forked processes use the same.
 func SetTempDir(path string) (err error) {
 	var tempDir string
